@@ -23,7 +23,7 @@ __copyright__ = '(C) 2015, Victor Olaya'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '58734527abdbaab95719a48a4a64ea05abd70b44'
+__revision__ = '93e761dd2396af21db9bb93e26d7cba680c906ca'
 
 from qgis.PyQt.QtCore import QCoreApplication
 from qgis.PyQt.QtWidgets import (QWidget,
@@ -142,3 +142,5 @@ class GdalParametersPanel(ParametersPanel):
             self.text.setPlainText(" ".join(commands))
         except AlgorithmDialogBase.InvalidParameterValue as e:
             self.text.setPlainText(self.tr("Invalid value for parameter '{0}'").format(e.parameter.description()))
+        except AlgorithmDialogBase.InvalidOutputExtension as e:
+            self.text.setPlainText(e.message)
