@@ -23,7 +23,7 @@ __copyright__ = '(C) 2012, Victor Olaya'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '4a4b62ed19d2333b1f3a9e72bf77119048e3c9c0'
+__revision__ = '93e761dd2396af21db9bb93e26d7cba680c906ca'
 
 import math
 
@@ -96,7 +96,8 @@ class ModelerParameterDefinitionDialog(QDialog):
         super(ModelerParameterDefinitionDialog, self).closeEvent(event)
 
     def setupUi(self):
-        self.setWindowTitle(self.tr('Parameter Definition'))
+        type_metadata = QgsApplication.processingRegistry().parameterType(self.param.type() if self.param else self.paramType)
+        self.setWindowTitle(self.tr('{} Parameter Definition').format(type_metadata.name()))
         self.setMinimumWidth(300)
 
         self.verticalLayout = QVBoxLayout(self)
@@ -209,6 +210,34 @@ class ModelerParameterDefinitionDialog(QDialog):
             self.verticalLayout.addWidget(self.datatypeCombo)
         elif (self.paramType == parameters.PARAMETER_NUMBER or self.paramType == parameters.PARAMETER_DISTANCE or
               isinstance(self.param, (QgsProcessingParameterNumber, QgsProcessingParameterDistance))):
+
+            if (self.paramType == parameters.PARAMETER_DISTANCE or
+                    isinstance(self.param, QgsProcessingParameterDistance)):
+                self.verticalLayout.addWidget(QLabel(self.tr('Linked input')))
+                self.parentCombo = QComboBox()
+                self.parentCombo.addItem('', '')
+                idx = 1
+                for param in list(self.alg.parameterComponents().values()):
+                    definition = self.alg.parameterDefinition(param.parameterName())
+                    if isinstance(definition, (QgsProcessingParameterFeatureSource,
+                                               QgsProcessingParameterVectorLayer,
+                                               QgsProcessingParameterMapLayer,
+                                               QgsProcessingParameterCrs)):
+                        self.parentCombo.addItem(definition.description(), definition.name())
+                        if self.param is not None:
+                            if self.param.parentParameterName() == definition.name():
+                                self.parentCombo.setCurrentIndex(idx)
+                        idx += 1
+                self.verticalLayout.addWidget(self.parentCombo)
+            else:
+                self.verticalLayout.addWidget(QLabel(self.tr('Number type')))
+                self.type_combo = QComboBox()
+                self.type_combo.addItem(self.tr('Float'), QgsProcessingParameterNumber.Double)
+                self.type_combo.addItem(self.tr('Integer'), QgsProcessingParameterNumber.Integer)
+                if self.param:
+                    self.type_combo.setCurrentIndex(self.type_combo.findData(self.param.dataType()))
+                self.verticalLayout.addWidget(self.type_combo)
+
             self.verticalLayout.addWidget(QLabel(self.tr('Min value')))
             self.minTextBox = QLineEdit()
             self.verticalLayout.addWidget(self.minTextBox)
@@ -224,7 +253,7 @@ class ModelerParameterDefinitionDialog(QDialog):
             if self.param is not None:
                 default = self.param.defaultValue()
                 if self.param.dataType() == QgsProcessingParameterNumber.Integer:
-                    default = int(math.floor(default))
+                    default = int(math.floor(float(default)))
                 if default:
                     self.defaultTextBox.setText(str(default))
             self.verticalLayout.addWidget(self.defaultTextBox)
@@ -399,11 +428,36 @@ class ModelerParameterDefinitionDialog(QDialog):
             self.param = QgsProcessingParameterMultipleLayers(
                 name, description,
                 self.datatypeCombo.currentData())
+        elif (self.paramType == parameters.PARAMETER_DISTANCE or
+              isinstance(self.param, QgsProcessingParameterDistance)):
+            self.param = QgsProcessingParameterDistance(name, description,
+                                                        self.defaultTextBox.text())
+            try:
+                vmin = self.minTextBox.text().strip()
+                if not vmin == '':
+                    self.param.setMinimum(float(vmin))
+                vmax = self.maxTextBox.text().strip()
+                if not vmax == '':
+                    self.param.setMaximum(float(vmax))
+            except:
+                QMessageBox.warning(self, self.tr('Unable to define parameter'),
+                                    self.tr('Wrong or missing parameter values'))
+                return
+
+            if self.parentCombo.currentIndex() < 0:
+                QMessageBox.warning(self, self.tr('Unable to define parameter'),
+                                    self.tr('Wrong or missing parameter values'))
+                return
+            parent = self.parentCombo.currentData()
+            if parent:
+                self.param.setParentParameterName(parent)
         elif (self.paramType == parameters.PARAMETER_NUMBER or
-              isinstance(self.param, (QgsProcessingParameterNumber, QgsProcessingParameterDistance))):
+              isinstance(self.param, QgsProcessingParameterNumber)):
+
+            type = self.type_combo.currentData()
+            self.param = QgsProcessingParameterNumber(name, description, type,
+                                                      self.defaultTextBox.text())
             try:
-                self.param = QgsProcessingParameterNumber(name, description, QgsProcessingParameterNumber.Double,
-                                                          self.defaultTextBox.text())
                 vmin = self.minTextBox.text().strip()
                 if not vmin == '':
                     self.param.setMinimum(float(vmin))
